import unittest
import os
import os.path
import re
import sys
import tempfile
import json

from kogu import Kogu, KoguException
try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO

import io
class KoguTests(unittest.TestCase):

    def setUp(self):
        Kogu.verbose = True

    def tearDown(self):
        sys.stdout = sys.__stdout__
        sys.stdin = sys.__stdin__
        # clear away any previous parameters.json
        self._parameters_file_name()

    def _error_names(self, missing=True):
        invalid = ["_a4", "1a", "!a", "b$b", ":F", ",",
                   "'", "1", "0.5", "-", "A B", "A-", 1, 0.5]
        empty = ["", " ", None]
        if missing:
            invalid.extend(empty)
        return invalid

    def _clean(self, s):
        s.truncate(0)
        s.seek(0)

    def _assert_warn(self, value, msg=None):
        self._assert_line_format("Warning! Kogu", value, msg)

    def _assert_line_format(self, linestart, value, msg=None):
        regex = r"^" + re.escape(linestart) + r": .*\n$"
        self.assertRegexpMatches(value, regex, msg)

    def test_comment(self):
        sys.stdout = stdout = StringIO()
        Kogu.comment("This is comment")
        k = json.loads(stdout.getvalue())
        self.assertEquals("This is comment", k["comment"])
        self._clean(stdout)

    def test_comment_linebreaks(self):
        sys.stdout = stdout = StringIO()
        Kogu.comment("This\r\nis\ncomment\r")
        k = json.loads(stdout.getvalue())
        self.assertEquals("This is comment", k["comment"])
        self._clean(stdout)

    def test_comment_empty(self):
        sys.stdout = stdout = StringIO()
        Kogu.comment("")
        self._assert_warn(stdout.getvalue())
        self._clean(stdout)

    def test_comment_numeric(self):
        sys.stdout = stdout = StringIO()
        Kogu.comment(123)
        k = json.loads(stdout.getvalue())
        self.assertEquals("123", k["comment"])        
        self._clean(stdout)

    def test_name(self):
        sys.stdout = stdout = StringIO()
        name = "nEw (nAme)!"
        Kogu.name(name)
        k = json.loads(stdout.getvalue())
        self.assertEquals(name, k["name"])
        self._clean(stdout)

    def test_name_empty(self):
        sys.stdout = stdout = StringIO()
        Kogu.name("")
        self._assert_warn(stdout.getvalue())
        self._clean(stdout)

    def test_tag(self):
        sys.stdout = stdout = StringIO()
        tag = "nEw (tAg)!"
        Kogu.tag(tag)
        k = json.loads(stdout.getvalue())
        self.assertEquals(tag, k["tag"])
        self._clean(stdout)

    def test_tag_empty(self):
        sys.stdout = stdout = StringIO()
        Kogu.tag("")
        self._assert_warn(stdout.getvalue())
        self._clean(stdout)

    def test_untag(self):
        sys.stdout = stdout = StringIO()
        tag = "nEw (tAg)!"
        Kogu.untag(tag)
        k = json.loads(stdout.getvalue())
        self.assertEquals(tag, k["untag"])
        self._clean(stdout)

    def test_untag_empty(self):
        sys.stdout = stdout = StringIO()
        Kogu.untag("")
        self._assert_warn(stdout.getvalue())
        self._clean(stdout)

    def test_plot_series_string(self):
        sys.stdout = stdout = StringIO()
        Kogu.plot("line", "x")
        k = json.loads(stdout.getvalue())
        self.assertEquals("line", k["plot"]["type"])
        self.assertIn("x", k["plot"]["metrics"])
        self._clean(stdout)

    def test_plot_series_array(self):
        sys.stdout = stdout = StringIO()
        Kogu.plot("line", ["x", "y"])
        k = json.loads(stdout.getvalue())
        self.assertIn("x", k["plot"]["metrics"])
        self.assertIn("y", k["plot"]["metrics"])
        self._clean(stdout)

    def test_plot_series_array_single(self):
        sys.stdout = stdout = StringIO()
        Kogu.plot("line", ["x"])
        k = json.loads(stdout.getvalue())
        self.assertIn("x", k["plot"]["metrics"])
        self._clean(stdout)

    def test_plot_series_array_duplicates(self):
        sys.stdout = stdout = StringIO()
        Kogu.plot("line", ["x", "x"])
        k = json.loads(stdout.getvalue())
        self.assertIn("x", k["plot"]["metrics"])
        self.assertEquals(1, len(k["plot"]["metrics"]))
        self._clean(stdout)

    def test_plot_series_set(self):
        sys.stdout = stdout = StringIO()
        Kogu.plot("line", set(["x", "x", "y"]))
        k = json.loads(stdout.getvalue())
        self.assertIn("x", k["plot"]["metrics"])
        self.assertEquals(2, len(k["plot"]["metrics"]))
        self._clean(stdout)

    def test_plot_series_invalid(self):
        sys.stdout = stdout = StringIO()
        for name in self._error_names(missing=False): # allow empty metrics in this case
            Kogu.plot("line", name)
            self._assert_warn(
                stdout.getvalue(), "Metric '%s' should not be allowed" % name)
            self._clean(stdout)

    def test_plot_options(self):
        sys.stdout = stdout = StringIO()
        Kogu.plot("line", ['test'], "Test Plot", "Y-label")
        k = json.loads(stdout.getvalue())
        self.assertEquals("line", k["plot"]["type"])
        self.assertEquals("Y-label", k["plot"]["y_label"])
        self.assertEquals("Test Plot", k["plot"]["name"])
        self.assertIn("test", k["plot"]["metrics"])
        self._clean(stdout)

    def test_plot_options_default(self):
        sys.stdout = stdout = StringIO()
        Kogu.plot("line", "test")
        k = json.loads(stdout.getvalue())
        self.assertEquals("line", k["plot"]["type"])
        self.assertEquals("Y", k["plot"]["y_label"])
        self.assertEquals("line", k["plot"]["name"])
        self.assertIn("test", k["plot"]["metrics"])        
        self._clean(stdout)

    def test_plot_options_empty(self):
        sys.stdout = stdout = StringIO()
        Kogu.plot("", "")
        self._assert_warn(stdout.getvalue())
        self._clean(stdout)

    def test_plot_options_double_quotes(self):
        sys.stdout = stdout = StringIO()
        Kogu.plot("line", ['test'], "Test 'Plot")
        k = json.loads(stdout.getvalue())
        self.assertEquals("Test 'Plot", k["plot"]["name"])
        self.assertIn("test", k["plot"]["metrics"])
        self._clean(stdout)

    def test_upload(self):
        sys.stdout = stdout = StringIO()
        with tempfile.NamedTemporaryFile() as fp:
            Kogu.upload(fp.name)
            k = json.loads(stdout.getvalue())
            self.assertEquals(fp.name, k["upload"])
        self._clean(stdout)

    def test_upload_non_existing(self):
        sys.stdout = stdout = StringIO()
        path = "/non/existing/file"
        Kogu.upload(path)
        k = json.loads(stdout.getvalue())
        self.assertEquals(path, k["upload"])        
        self._clean(stdout)

    def test_upload_empty(self):
        sys.stdout = stdout = StringIO()
        Kogu.upload("")
        self._assert_warn(stdout.getvalue(), "parameters not passed")
        self._clean(stdout)    

    def test_metrics_none(self):
        sys.stdout = stdout = StringIO()
        Kogu.metrics(None)
        self._assert_warn(stdout.getvalue(), "None is not a valid metrics dictionary")
        self._clean(stdout)

    def test_metrics_empty(self):
        sys.stdout = stdout = StringIO()
        Kogu.metrics({})
        self.assertFalse(stdout.getvalue(), "Got->["+stdout.getvalue()+"] but expected empty string")
        self._clean(stdout)

    def test_metrics_none_key_and_value(self):
        sys.stdout = stdout = StringIO()
        Kogu.metrics({None: None})
        self._assert_warn(stdout.getvalue(), "None is not valid key value")
        self._clean(stdout)

    def test_metrics_none_key(self):
        sys.stdout = stdout = StringIO()
        Kogu.metrics({None: 3})
        self._assert_warn(stdout.getvalue(), "None is not valid key value")
        self._clean(stdout)

    def test_metrics_none_iteration(self):
        sys.stdout = stdout = StringIO()
        Kogu.metrics({"x": 3}, None)
        self._assert_warn(stdout.getvalue(), "None is not a valid iteration value")
        self._clean(stdout)

    def test_metrics_string_iteration(self):
        sys.stdout = stdout = StringIO()
        Kogu.metrics({"x": 3}, "string")
        self._assert_warn(stdout.getvalue(), "a string is not a valid iteration value")
        self._clean(stdout)

    def test_metrics_float_iteration(self):
        sys.stdout = stdout = StringIO()
        Kogu.metrics({"x": 3}, 3.14)
        self._assert_warn(stdout.getvalue(), "a floating point number is not a valid iteration value")
        self._clean(stdout)

    def test_metrics_empty_object_iteration(self):
        sys.stdout = stdout = StringIO()
        Kogu.metrics({"x": 3}, {})
        self._assert_warn(stdout.getvalue(), "a dictionary is not a valid iteration value")
        self._clean(stdout)

    def test_metrics_invalid_key(self):
        sys.stdout = stdout = StringIO()
        for name in self._error_names():
            Kogu.metrics({name: "3"})
            self._assert_warn(stdout.getvalue(), "Processing '%s' ->" % name)
            self._clean(stdout)

    def test_metrics_iteration(self):
        sys.stdout = stdout = StringIO()
        Kogu.metrics({"A": "3"}, 3)
        k = json.loads(stdout.getvalue())
        self.assertEquals("3", k["metric"]["metrics"]["A"])        
        self.assertEquals(3, k["metric"]["iteration"])        
        self._clean(stdout)

    def test_metrics_iteration_default(self):
        sys.stdout = stdout = StringIO()
        Kogu.metrics({"A": "3"})
        k = json.loads(stdout.getvalue())
        self.assertEquals("3", k["metric"]["metrics"]["A"])        
        self.assertEquals(-1, k["metric"]["iteration"])                
        self._clean(stdout)

    def _test_metrics_values_string(self):
        sys.stdout = stdout = StringIO()
        Kogu.metrics({"A": "3", "B": "ok"})
        k = json.loads(stdout.getvalue())
        self.assertEquals("3", k["metric"]["metrics"]["A"])        
        self.assertEquals("ok", k["metric"]["metrics"]["B"])        
        self._clean(stdout)

    def _test_metrics_values_int(self):
        sys.stdout = stdout = StringIO()
        Kogu.metrics({"A": 3, "B": 6})
        k = json.loads(stdout.getvalue())
        self.assertEquals(3, k["metric"]["metrics"]["A"])        
        self.assertEquals(6, k["metric"]["metrics"]["B"])        
        self._clean(stdout)

    def test_metrics_values_float(self):
        sys.stdout = stdout = StringIO()
        float_value = 1.0 / 3.0
        Kogu.metrics({"A": float_value})
        k = json.loads(stdout.getvalue())
        self.assertEquals(float_value, k["metric"]["metrics"]["A"])        
        self._clean(stdout)

    def test_metrics_values_bool(self):
        sys.stdout = stdout = StringIO()
        Kogu.metrics({"b": True, "a": False})
        k = json.loads(stdout.getvalue())
        self.assertTrue(k["metric"]["metrics"]["b"])        
        self.assertFalse(k["metric"]["metrics"]["a"])        
        self._clean(stdout)

    def test_metrics_keys_strip_whitespace(self):
        sys.stdout = stdout = StringIO()
        Kogu.metrics({" A ": 3})
        k = json.loads(stdout.getvalue())
        self.assertEquals(3, k["metric"]["metrics"]["A"])        
        self._clean(stdout)

    def test_metrics_keys_whitespace_duplicates(self):
        sys.stdout = stdout = StringIO()
        Kogu.metrics({" B": 6, "B ": 9})
        k = json.loads(stdout.getvalue())
        self.assertEquals(9, k["metric"]["metrics"]["B"])
        self.assertNotEquals(6, k["metric"]["metrics"]["B"])    
        self._clean(stdout)

    def _test_metrics_keys_case_sensitive(self):
        sys.stdout = stdout = StringIO()
        Kogu.metrics({"b": 6, "B": 9})
        k = json.loads(stdout.getvalue())
        self.assertEquals(9, k["metric"]["metrics"]["B"])
        self.assertEquals(6, k["metric"]["metrics"]["b"])    
        self._clean(stdout)

    def test_load_parameters(self):
        params = '{"epochs":30}'

        path = os.path.join(tempfile.mkdtemp(), 'pipe')
        os.mkfifo(path)
        fd = os.open(path, os.O_RDWR|os.O_NONBLOCK)
        os.write(fd, params.encode())

        sys.stdin = stdin = io.open(fd, mode="rt")
        sys.stdout = stdout = StringIO()
        Kogu.load_parameters()

        self.assertEqual(epochs, 30)
        stdin.close()
        os.unlink(path)
        self._clean(stdout)

    def _parameters_file_name(self):
        json_path = os.path.join(sys.path[0], "parameters.json")
        try:
            os.remove(json_path)
        except OSError:
            pass
        return json_path

    def test_load_parameters_file(self):
        with open(self._parameters_file_name(), "w") as json_file:
            json_file.writelines('{"epochs":50}')
        sys.stdout = stdout = StringIO()
        Kogu.load_parameters()
        self.assertEqual(epochs, 50)
        self._clean(stdout)

    # Piped parameters overload parameters passed from
    # parameters json. Parameters available in parameters.json
    # are output via param lines
    def test_load_parameters_priority(self):       
        with open(self._parameters_file_name(), "w") as json_file:
            json_file.writelines(u'{"epochs":50,"spaces":25}')

        params = '{"epochs":10,"spaces": 22,"third":3}'

        path = os.path.join(tempfile.mkdtemp(), 'pipe')
        os.mkfifo(path)
        fd = os.open(path, os.O_RDWR|os.O_NONBLOCK)
        os.write(fd, params.encode())

        sys.stdin = stdin = io.open(fd, mode="rt")
        sys.stdout = stdout = StringIO()

        Kogu.load_parameters()

        self.assertEqual(epochs, 10)
        self.assertEqual(spaces, 22)
        self.assertEqual(third, 3)

        k = json.loads(stdout.getvalue())
        self.assertEquals(10, k["parameters"]["epochs"])
        self.assertEquals(22, k["parameters"]["spaces"])
        self.assertEquals(3, k["parameters"]["third"])

        stdin.close()
        os.unlink(path)
        self._clean(stdout)

    def test_load_parameters_file_invalid(self):
        with open(self._parameters_file_name(), "w") as json_file:
            json_file.writelines(u'epochs:55')
        sys.stdout = stdout = StringIO()
        self.assertRaises(KoguException, Kogu.load_parameters)
        self._clean(stdout)

    def test_update_parameters_values_string(self):
        sys.stdout = stdout = StringIO()
        Kogu.update_parameters({"A": "3", "B": "ok"})
        k = json.loads(stdout.getvalue())
        self.assertEquals("3", k["parameters"]["A"])
        self.assertEquals("ok", k["parameters"]["B"])
        self._clean(stdout)

    def test_update_parameters_values_int(self):
        sys.stdout = stdout = StringIO()
        Kogu.update_parameters({"A": 3, "B": 6})
        k = json.loads(stdout.getvalue())
        self.assertEquals(3, k["parameters"]["A"])
        self.assertEquals(6, k["parameters"]["B"])
        self._clean(stdout)

    def test_update_parameters_values_float(self):
        sys.stdout = stdout = StringIO()
        fl = 1.0 / 3.0
        Kogu.update_parameters({"A": fl})
        k = json.loads(stdout.getvalue())
        self.assertEquals(fl, k["parameters"]["A"])
        self._clean(stdout)

    def test_update_parameters_values_bool(self):
        sys.stdout = stdout = StringIO()
        Kogu.update_parameters({"b": True, "a": False})
        k = json.loads(stdout.getvalue())
        self.assertTrue(k["parameters"]["b"])
        self.assertFalse(k["parameters"]["a"])
        self._clean(stdout)

    def test_update_parameters_keys_strip_whitespace(self):
        sys.stdout = stdout = StringIO()
        Kogu.update_parameters({" A ": 3})
        k = json.loads(stdout.getvalue())
        self.assertEquals(3, k["parameters"]["A"])
        self._clean(stdout)

    def test_update_parameters_keys_whitespace_duplicates(self):
        sys.stdout = stdout = StringIO()
        Kogu.update_parameters({" B": 6, "B ": 9})
        k = json.loads(stdout.getvalue())
        self.assertEquals(9, k["parameters"]["B"])
        self.assertNotEquals(6, k["parameters"]["B"])
        self._clean(stdout)

    def test_update_parameters_keys_case_sensitive(self):
        sys.stdout = stdout = StringIO()
        Kogu.update_parameters({"b": 6, "B": 9})
        k = json.loads(stdout.getvalue())
        self.assertEquals(9, k["parameters"]["B"])
        self.assertEquals(6, k["parameters"]["b"])
        self._clean(stdout)

    def test_update_parameters_keys_invalid(self):
        sys.stdout = stdout = StringIO()
        for name in self._error_names():
            Kogu.update_parameters({name: "3"})
            self._assert_warn(
                stdout.getvalue(), "%s should not be valid identifier" % name)
            self._clean(stdout)

    def test_fail(self):
        sys.stdout = stdout = StringIO()
        reason = "some, reason)"
        Kogu.fail(reason)
        k = json.loads(stdout.getvalue())
        self.assertEquals(reason, k["fail"])
        self._clean(stdout)

    def test_fail_reason_empty(self):
        sys.stdout = stdout = StringIO()
        Kogu.fail()
        k = json.loads(stdout.getvalue())
        self.assertEquals("", k["fail"])
        self._clean(stdout)

if __name__ == '__main__':
    unittest.main()



"""Implementation of Kogu project helper library"""
from __future__ import print_function
import os
import sys
import json
import re
import inspect
from stat import S_ISFIFO

class KoguException(Exception):
    """
    KoguException is custom exception class used by the library
    """
    pass

class Kogu(object):
    """
    Kogu is class that implements helper methods for reading input parameters and
    sending output parameters to Kogu server
    """

    verbose = False

    @classmethod
    def load_parameters(cls, output=False):
        """
        Loads parameters to calling module dictionary as variables. Parameters can be passed
        as JSON via stdin pipe and/or via parameters.json file located at script folder.
        The parameters passed via pipe overwrite those passed from parameters.json file.
        Arguments
        ---------
        * **output** - Flag indicating whether to print the parameters also to stdout.
        """

        # get caller dictionary to load variables into
        frm = inspect.stack()[1]
        mod = inspect.getmodule(frm[0])
        module_name = sys.modules[mod.__name__]

        dic = {}

        # load parameters from parameters.json file located in the same dir with the script file
        # the file should contain json e.g.: { "dx": 0.03, "n": 4 }
        try:
            with open(os.path.join(sys.path[0], "parameters.json")) as data_file:
                dic = json.load(data_file)
                vars(module_name).update(dic)
        except IOError:
            pass
        except ValueError as err:
            raise KoguException('parameters.json cannot be parsed: ' + str(err))

        # load parameters from json string from standard input (in case stdin a fifo (named-pipe))
        if S_ISFIFO(os.fstat(sys.stdin.fileno()).st_mode):
            try:
                stdin = ''.join(sys.stdin.readlines())
                content = json.loads(stdin)
                dic.update(content)
                vars(module_name).update(dic)
            except (IOError, ValueError) as err:
                pass

        if dic:
            cls.update_parameters(dic, output)

    @classmethod
    def _validate_dict(cls, dic):
        if not all(cls._validate_metric_key(k) for k in dic):
            raise KoguException('invalid parameter key in: %s' % dic)
        
        return {k.strip():v for (k,v) in dic.items()}
        

    @classmethod
    def update_parameters(cls, dic, output=False):
        """
        Creates Kogu specific parameter registration line to calling script stdout.
        You can call the method multiple times for the same parameters. In this case the
        later calls take precedence.
        Arguments
        ---------
        * **dic** - Dictionary of parameter key-value pairs. Keys must be valid identifiers.
        * **output** - Flag indicating whether to print the parameters also to stdout.
        """
        try:
            d = cls._validate_dict(dic)            
            cls._json_dumps({"parameters": d})

        except KoguException as ex:
            cls._warn(ex)

        if output:
            print("Parameters:")
            for k in dic:
                print("  {0}: {1}".format(k, dic[k]))

    @classmethod
    def metrics(cls, dic, iteration=-1):
        """
        Creates Kogu specific metric information line to calling script stdout.
        You can call the method multiple times for the same metrics. In this case the later
        calls take precedence.
        Arguments
        ---------
        * **dic** - key-value pairs of metric names (must be valid identifiers) and values.
        * **iteration** - Optional iteration number (integer) to be logged with the metrics.
                In case omitted sent metric is considered to be result metric.
        """
        try:
            if not isinstance(dic, dict):
                raise KoguException('dic should be a dictionary')

            if not isinstance(iteration, int):
                raise KoguException('iteration should be a number')

            d = cls._validate_dict(dic)

            if len(d) > 0:
                cls._json_dumps({"metric": {"iteration": iteration, "metrics": d}})

        except KoguException as ex:
            cls._warn(ex)

    @classmethod
    def comment(cls, comment):
        """
        Creates Kogu specific comment line to calling script stdout. Newlines (\\n) are replaced
        with spaces. Carriage returns (\\r) are stripped.
        Arguments
        ---------
        * **comment** - Text to add as a comment.
        """
        try:
            if not comment:
                raise KoguException('comment must be passed')

            # strip all line breaks
            comment = '{}'.format(comment)
            comment = comment.replace('\n', ' ').replace('\r', '')

            cls._json_dumps({"comment": comment})
        except KoguException as ex:
            cls._warn(ex)

    @classmethod
    def name(cls, name):
        """
        Creates Kogu specific name line to calling script stdout.
        Arguments
        ---------
        * **name** - Text to set as a name.
        """
        try:
            if not name:
                raise KoguException('name must be passed')

            cls._json_dumps({"name": name})
        except KoguException as ex:
            cls._warn(ex)

    @classmethod
    def tag(cls, tag):
        """
        Creates Kogu specific tag line to calling script stdout.
        Arguments
        ---------
        * **tag** - Text to add as a tag.
        """
        try:
            if not tag:
                raise KoguException('tag must be passed')

            cls._json_dumps({"tag": tag})
        except KoguException as ex:
            cls._warn(ex)

    @classmethod
    def untag(cls, tag):
        """
        Creates Kogu specific untag line to calling script stdout.
        Arguments
        ---------
        * **tag** - Text of tag to remove.
        """
        try:
            if not tag:
                raise KoguException('tag must be passed')

            cls._json_dumps({"untag": tag})
        except KoguException as ex:
            cls._warn(ex)

    @classmethod
    def upload(cls, filename):
        """
        Creates Kogu specific File upload line to calling script stdout. The line is
        created only if the referenced file exists.
        Arguments
        ---------
        * **filename** - Relative or absolute path to the file. In case of relative path
        is passed it is evaluated from script root.
        """
        try:
            cls._check_file(filename, should_exist=False)
            cls._json_dumps({"upload": filename})
        except KoguException as ex:
            cls._warn(ex)

    @classmethod
    def plot(cls, plot_type, series, name=None, y_label=None):
        """
        Creates Kogu specific Plot registration line to calling script stdout.
        Arguments
        ---------
        * **plot_type** - String. Type of the plot to be used.
        * **series** - Optional list of metrics to be used for the plot. Metric names must be valid
        identifiers.
        * **name** - Optional name of the plot. When omitted it will default to plot_type.
        * **y_label** - Optional label for the Y-axis of the plot. When omitted will default to Y.
        """
        try:
            if not plot_type:
                raise KoguException('plot_type must be passed')

            if not series:
                raise KoguException('series must be passed')

            if not name:
                name = plot_type

            if not y_label:
                y_label = "Y"

            cls._json_dumps({
                "plot": {
                    "type": plot_type, 
                    "name": name, 
                    "y_label": y_label, 
                    "metrics": cls._process_metrics(series),
                    }
                })
        except KoguException as ex:
            cls._warn(ex)

    @classmethod
    def fail(cls, reason=None):
        """
        Creates Kogu specific fail line to calling script stdout.
        Arguments
        ---------
        * **reason** - Optional text to use as a failure reason.
        """
        try:
            reason = "" if reason is None else reason
            cls._json_dumps({"fail": reason})
        except KoguException as ex:
            cls._warn(ex)

    @classmethod
    def _cleanupjson(cls, j):
        for key, value in list(j.items()):
            if value is None:
                del j[key]
            elif isinstance(value, dict):
                cls._cleanupjson(value)
        return j

    @classmethod
    def _json_dumps(cls, obj, cleanup=True):
        o = cls._cleanupjson(obj) if cleanup else obj
        cls._print("{}", json.dumps(o))
    
    @classmethod
    def _warn(cls, ex):
        # TODO: It would be nice to have exception context logged i.e. calling method name
        # maybe inspect.trace() would help
        cls._print("Warning! Kogu: {}", str(ex))

    @classmethod
    def _print(cls, fmt, *args):
        if cls.verbose or not sys.stdin.isatty():
            if isinstance(fmt, str):
                print(fmt.format(*args))

    @classmethod
    def _check_file(cls, filename, should_exist=True):

        if not filename:
            raise KoguException('filename must be passed')

        if not os.path.isabs(filename):
            filename = os.path.join(sys.path[0], filename)

        if should_exist and not os.path.exists(filename):
            raise KoguException('File %s does not exist' % filename)

        return filename

    @classmethod
    def _is_string(cls, obj):
        try:
            return isinstance(obj, basestring) # python 2
        except NameError:
            return isinstance(obj, str) # python 3

    @classmethod
    def _validate_metric_key(cls, key):
        if cls._is_string(key):
            key = key.strip()
            if key:
                return re.match(r"^[a-zA-Z]\w*$", key)
        return False

    @classmethod
    def _process_metrics(cls, metrics):
        if metrics:
            if isinstance(metrics, set):
                pass
            elif isinstance(metrics, list):
                metrics = set(metrics)
            else:
                metrics = set([metrics])

            if not all(cls._validate_metric_key(n) for n in metrics):
                raise KoguException('invalid metric values passed: %s' % metrics)

            metrics = set([x.strip() for x in metrics])

        return list(metrics)


import os
from setuptools import setup, find_packages

def _get_version():
    dir_name = os.path.dirname(os.path.abspath(__file__))
    with open(os.path.join(dir_name, 'VERSION')) as version_file:
        return version_file.read().strip()

setup(
    name='kogu',
    version=_get_version(),
    description='Kogu helper library',
    url='https://kogu.io',
    author='Proekspert AS',
    author_email='hello@kogu.io',
    license='MIT',
    packages=find_packages(),
    python_requires='>=2.7',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.6',
    ],
)
